/* v0.8
 *
 * hashtab.c:  Basic hash table code.
 *
 * This program is free software and may be freely redistributed as
 * specified in the GNU General Public License.  Please see the file
 * 'COPYING' for details.
 */

#include <stdlib.h>
#include <string.h>

#include "hashtab.h"

enum search_type {NEAR, EXACT};

HNODE *modified_search(char *, HNODE *, int);

HNODE *modified_search(char *skey, HNODE *table, int type)
{
	int d;
	HNODE *tmp=NULL, *ptr=table;

	while (ptr != NULL) {

		tmp = ptr;

		if ((d=strcasecmp(skey, ptr->key))==0) return ptr;
		if (d < 0)
			ptr=ptr->left_child;
		else
			ptr=ptr->right_child;

	}
	return((type == NEAR) ? tmp: NULL);
}

int *hashSearch(char *skey, HNODE *table)
{
	HNODE *ptr;

	ptr = modified_search(skey, table, EXACT);

	if (ptr != NULL)
		return(ptr->data);
	else
		return(NULL);

}

void hashInsert(char *skey, int *data, HNODE **table)
{
	HNODE *newent, *tmp;
	int d;

	if ((newent = (HNODE *)malloc(sizeof(HNODE)))==NULL)
		return;

	newent->key = skey;
	newent->data = data;
	newent->left_child = newent->right_child = NULL;

	if (*table != NULL) {

		tmp = modified_search(skey, *table, NEAR);
		d = strcasecmp(skey, tmp->key);
		if (d == 0)	/* exact match found */
			return;
		else if (d < 0) 
			tmp->left_child = newent;
		else
			tmp->right_child = newent;

	} 
	else
		*table = newent;

	return;

}		
